/* cc -I/usr/openwin/include -omandelbrot mandelbrot.c -L/usr/openwin/lib -lX11 -lm  */
/* Master Task of Mandel */


#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xos.h>
#include <stdio.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <stdlib.h>
#include <pvm3.h>

#define SLAVENAME "psmandelbrot"
#define PROC 4
#define NELEM 1000
#define	X_RESN	800       /* x resolution */
#define	Y_RESN	800       /* y resolution */

typedef struct complextype
	{
        float real, imag;
	} Compl;


main ()
{
	struct timeval tv1, tv2;	/* for timing */
	time_t t1,t2;
	Window		win;                            /* initialization for a window */
	unsigned
	int             width, height,                  /* window size */
                        x, y,                           /* window position */
                        border_width,                   /*border width in pixels */
                        display_width, display_height,  /* size of screen */
                        screen;                         /* which screen */

	char            *window_name = "Mandelbrot Set", *display_name = NULL;
	GC              gc;
	unsigned
	long		valuemask = 0;
	XGCValues	values;
	Display		*display;
	XSizeHints	size_hints;
	Pixmap		bitmap;
	XPoint		points[800];
	FILE		*fp, *fopen ();
	char		str[100];

   int mytid ;
   int tids[ PROC ] ;           /* slave task ids */
   int n = NELEM ;              /* number of data elements */
   int nproc = PROC ;           /* number of processes */
   int numt, l, who, msgtype ;
   int data[ NELEM ] ;          /* data array */
   int result[ PROC ] ;         /* array of results slaves*/
   int NumberOfElements;
   int Row, Column, Color;
   int totalpoints;

   XSetWindowAttributes attr[1];

   /* Mandlebrot variables */
   int i, j, k;
   Compl	z, c;
   float	lengthsq, temp;

   /* connect to Xserver */

   if (  (display = XOpenDisplay (display_name)) == NULL ) 
   {
   	fprintf (stderr, "drawon: cannot connect to X server %s\n",
				XDisplayName (display_name) );
	exit (-1);
    }

    /* get screen size */

	screen = DefaultScreen (display);
	display_width = DisplayWidth (display, screen);
	display_height = DisplayHeight (display, screen);

	/* set window size */

	width = X_RESN;
	height = Y_RESN;

	/* set window position */

	x = 0;
	y = 0;

        /* create opaque window */

	border_width = 4;
	win = XCreateSimpleWindow (display, RootWindow (display, screen),
				x, y, width, height, border_width,
				BlackPixel (display, screen), WhitePixel (display, screen));

	size_hints.flags = USPosition|USSize;
	size_hints.x = x;
	size_hints.y = y;
	size_hints.width = width;
	size_hints.height = height;
	size_hints.min_width = 300;
	size_hints.min_height = 300;

	XSetNormalHints (display, win, &size_hints);
	XStoreName(display, win, window_name);

        /* create graphics context */

	gc = XCreateGC (display, win, valuemask, &values);

	XSetBackground (display, gc, WhitePixel (display, screen));
	XSetForeground (display, gc, BlackPixel (display, screen));
	XSetLineAttributes (display, gc, 1, LineSolid, CapRound, JoinRound);

	attr[0].backing_store = Always;
	attr[0].backing_planes = 1;
	attr[0].backing_pixel = BlackPixel(display, screen);

	XChangeWindowAttributes(display, win, CWBackingStore | CWBackingPlanes | CWBackingPixel, attr);

	XMapWindow (display, win);
	XSync(display, 0);

   /*Create the processes*/
   mytid = pvm_mytid() ;
   numt = pvm_spawn( SLAVENAME, (char**)0, 0, "", nproc, tids ) ;
   if ( numt < nproc )
   {
      printf( "Trouble spawning slaves. Aborting. Error codes are: \n");
      for ( l=numt ; l<nproc ; l++ )
        printf( "TID %d %d\n", l, tids[l] ) ;
      for ( l = 0 ; l < numt ; l++ )
        pvm_kill( tids[l] ) ;
      pvm_exit() ;
      exit( 1 ) ;
    }
    NumberOfElements = X_RESN/nproc ;
    gettimeofday(&tv1, (struct timezone*)0);
    time(&t1);
    
    for(l=0;l<nproc;++l)
    {
      pvm_initsend(PvmDataDefault);
      pvm_pkint(tids,nproc,1);
      pvm_pkint(&NumberOfElements,1,1);
      pvm_send(tids[l],1);
    }
    totalpoints = X_RESN * Y_RESN;

    for(l=0; l < totalpoints ; ++l)
    {
      pvm_recv(-1,2);
      pvm_upkint(&Row,1,1);
      pvm_upkint(&Column,1,1);
      pvm_upkint(&Color,1,1);
      XSetForeground(display, gc, Color);
      XDrawPoint (display, win, gc, Column, Row);
    }

	gettimeofday(&tv2, (struct timezone*)0);
	printf("%8d uSec\n", (tv2.tv_sec - tv1.tv_sec) * 1000000 + tv2.tv_usec - tv1.tv_usec);
	time(&t2);
	printf("%d\n", difftime(t2,t1)); /* 1076232192 */

	XFlush (display);
	sleep (10);

	/* Program Finished */

}

